/*
 * HTTPARSER.C
 * Contains templates and parser fuctions.
 *
 * Copyright (C) 2009  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define ARGPARSER_C
#include "argparser.h"

/*
 * Program usage
 */
void usage(FILE *out, const char *error)
{
    fputs(usage_template, out);

    if(error)
        fputs(error, out);

    exit(1);
}

/*
 * A personalized strncat
 */
char *myncat(char *dest, const char *src, size_t n)
{
    size_t dest_len = strlen(dest);
    size_t i;

    for (i = 0 ; i < (n-dest_len-1) && src[i] != '\0' ; i++) {
        dest[dest_len + i] = src[i];
    }
    
    dest[dest_len + i] = '\0';

    return dest;
}


/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char* str, const char* regex) {
    regex_t re;

    // Compile the regex pattern.
    if (regcomp(&re, regex, REG_EXTENDED | REG_NOSUB) != 0) {
        fprintf(stderr, "Error on regex pattern compilation.\n");
        return -1;
    }

    // Use the pattern on the passed string
    if (regexec(&re, str, 0, NULL, 0))  {
        return 0;
    }

    regfree(&re);
    return 1;
}

/*
 * Read http request from a file
 */
int readhttprequest(struct iprange *ipr, char *path) {
    FILE *file;

    if((file=fopen(path, "r"))==NULL)
        usage(stderr, "\nError opening file for read the http request.\n");

    if(!fread(ipr->http_request, 1, MAX_DATA-1, file))
        usage(stderr, "\nError reading http request from file.\n");

    fclose(file);
    
    return 1;
}

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(struct iprange *ipr, int argc, char **argv)
{
    char ch;
    
    char range[MIN_ARG],  // ip range
         *request=NULL;   // http request
    
    int port = 80,        // port
        time = 3,         // child lifetime
        max_threads = 10, // max threads
        range_ext = 0,    // extended range (only ipv4)
        verbose = 0,      // verbose
        cookiejar = 0,    // cookie-jar output
        reqtype = 1,      // request type
        fingerprint = 0;  // fingerprint
    
    int ipver=0;   // ip version
    int single=0;  // single ip?
    
    struct addrinfo hints, *res=NULL;
    void *addr=NULL;
    
    // cleaning
    range[0]='\0';
    memset(ipr, 0, sizeof(struct iprange));
    memset(&hints, 0, sizeof(hints));
    
    const char *shortopt = "r:p:t:m:vjP:T:F"; // short options
    
    while ((ch = getopt (argc, argv, shortopt)) != -1)
        switch (ch) {
        
            case 'r': // IP range or dns name
                
                // ipv4 range
                if(regmatch(optarg, reg_ipv4_range)) {
                    ipver=VER_IPV4;
                    strncpy(range, optarg, MIN_ARG-1);
                }
                
                // ipv4 extended range
                else if(regmatch(optarg, reg_ipv4_range_ext)) {
                    ipver=VER_IPV4;
                    range_ext = 1;
                    strncpy(range, optarg, MIN_ARG-1);
                }
                
                // ipv6 range
                else if(regmatch(optarg, reg_ipv6_range)) {
                    ipver=VER_IPV6;
                    strncpy(range, optarg, MIN_ARG-1);
                }
                
                // dns name
                else if(getaddrinfo(optarg, NULL, &hints, &res) == 0) {
                
                    if (res->ai_family == AF_INET) { // IPv4
                        struct sockaddr_in *ipv4 = (struct sockaddr_in *) res->ai_addr;
                        addr = &(ipv4->sin_addr);
                        ipver=VER_IPV4;
                    }
                    
                    else if (res->ai_family == AF_INET6) { // IPv6
                        struct sockaddr_in6 *ipv6 = (struct sockaddr_in6 *)res->ai_addr;
                        addr = &(ipv6->sin6_addr);
                        ipver=VER_IPV6;
                    }

                    // convert the IP to a string and print it:
                    inet_ntop(res->ai_family, addr, range, MIN_ARG-1);
                    
                    single=1;
                    freeaddrinfo(res);
                }
                
                else {
                    usage(stderr, "\nArgument range is invalid.\n");
                }

                break;
            
            case 'p': // Port
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument port is invalid.\n");
                port = atoi(optarg);
                break;
            
            case 't': // Time
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument time is invalid.\n");
                time = atoi(optarg);
                break;
            
            case 'm': // Max threads
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument max threads is invalid.\n");
                max_threads = atoi(optarg);
                break;   
            
            case 'v': // Full answer
                verbose = 1;
                break;
            
            case 'j': // Cookie jar
                cookiejar = 1;
                break;
            
            case 'P': // Payload file
                request = optarg;
                break;
            
            case 'T': // Request type
                if(!strcasecmp(optarg, "get")) {
                    reqtype = GET_REQ;
                }
                else if(!strcasecmp(optarg, "head")) {
                    reqtype = HEAD_REQ;
                }
                else if(!strcasecmp(optarg, "get_not")) {
                    reqtype = GET_NOT_REQ;
                }
                else if(!strcasecmp(optarg, "get_wrong")) {
                    reqtype = GET_WRONG_REQ;
                }
                else if(!strcasecmp(optarg, "options")) {
                    reqtype = OPTIONS_REQ;
                }
                /*else if(!strcasecmp(optarg, "put")) {
                    reqtype = PUT_REQ;
                }*/
                else if(!strcasecmp(optarg, "delete")) {
                    reqtype = DELETE_REQ;
                }
                else {
                    usage(stderr, "\nArgument request type is invalid.\n");
                }
                break;
            
            case 'F': // Enable fingerprinting
                fingerprint = 1;
                break;
                            
            case '?':
            default:
                usage(stderr, NULL);
        }


    /* Check collected args */
    if(range[0]=='\0') {
        usage(stderr, "\nWhere is the range? Uff...\n");
    }
    
    if(time<1) time = 3;

    if(max_threads<1) max_threads = 10;
    
    if(port<1) port = 80;
    
    
    /* Fill iprange structure */
    
    // port
    ipr->port=port;
    
    // time
    ipr->time=time;    
    
    // max processes
    ipr->max_threads=max_threads;
    
    // extended range
    ipr->range_ext=range_ext;
    
    // verbose
    ipr->verbose=verbose;
    
    // cookiejar
    ipr->cookiejar=cookiejar;
    
    // request type
    ipr->reqtype=reqtype;
    
    // fingerprint
    ipr->fingerprint=fingerprint;
    
    // ipversion
    ipr->ipver=ipver;

    // range
	if(range[0]!='\0') {
        char *c, *x, *z;
	
        strncpy(ipr->range, range, MIN_ARG-1);
        
        // ipv4
        if(ipver==VER_IPV4) {
            c = range;
            x = range;

            if(range_ext) {
                while(*c!='-') c++;
                while(*c!='.') c--;
                c--;
                while(*c!='.') c--;
            }
            
            else {
                while(*c!='\0') c++;
                while(*c!='.') c--;
            }
            
            *c = '\0';
            strncpy(ipr->block, range, MIN_ARG-1);
            *c = '.';

            // start
            if(!single) {
                while(*x!='-') x++;
                *x='\0';
            }
            
            c++;
            
            // extended range
            if(range_ext) {
                z=c;
                while(*z!='.') z++;
                
                *z='\0';
                ipr->start2=atoi(c);
                *z='.';
                
                while(*c!='.') c++;
                c++;
                
                ipr->start=atoi(c);               
            }
            
            // range
            else {
                ipr->start=atoi(c);
            }
    
            // single ip
            if(single) {
                strcat(ipr->range,"-");
                strcat(ipr->range,c);
                ipr->end=ipr->start;
            }
            
            // extended range
            else if(range_ext) {
                x++;
                z=x;
                while(*z!='.') z++;
                
                *z='\0';
                ipr->end2=atoi(x);
                *z='.';
                
                while(*x!='.') x++;
                x++;
                ipr->end=atoi(x);       
            }
            
            // range
            else {
                x++;
                ipr->end=atoi(x);
            }
        }
        
        // ipv6
        else if(ipver==VER_IPV6) {
            c = range;
            x = range;

            while(*c!='\0') c++;
            while(*c!=':') c--;

            *c = '\0';
            strncpy(ipr->block, range, MIN_ARG-1);
            *c = ':';

            // start
            if(!single) {
                while(*x!='-') x++;
                *x='\0';
            }
            
            c++;
            
            sscanf(c, "%x", &ipr->start);

            // single ip
            if(single) {
                strcat(ipr->range,"-");
                strcat(ipr->range,c);
                ipr->end=ipr->start;
            }
            
            // range
            else {
                x++;
                sscanf(x, "%x", &ipr->end);
            }
        }
	}
   
    // check range
    if(ipr->end < ipr->start && range_ext==0)
        usage(stderr, "\nArgument ip-range is impossible ;)\n");

    else if(ipr->end2 < ipr->start2 && range_ext==1)
        usage(stderr, "\nArgument ip-range is impossible ;)\n");
    
    //http_request
    if(request) {
        readhttprequest(ipr, request);
    }
    else {
        if(reqtype == GET_REQ) {
            strncpy(ipr->http_request, get_request, MAX_DATA-1);
        }
        else if(reqtype == HEAD_REQ) {
            strncpy(ipr->http_request, head_request, MAX_DATA-1);
        }
        else if(reqtype == GET_NOT_REQ) {
            strncpy(ipr->http_request, get_not_exist_request, MAX_DATA-1);
        }
        else if(reqtype == GET_WRONG_REQ) {
            strncpy(ipr->http_request, get_wrong_version_request, MAX_DATA-1);
        }
        else if(reqtype == OPTIONS_REQ) {
            strncpy(ipr->http_request, options_request, MAX_DATA-1);
        }
        /*else if(reqtype == PUT_REQ) {
            strncpy(ipr->http_request, put_request, MAX_DATA-1);
        }*/
        else if(reqtype == DELETE_REQ) {
            strncpy(ipr->http_request, delete_request, MAX_DATA-1);
        }
    }
    
    return 1;
}


