/*
 * HTTPARSER.H
 * Contains templates and parser fuctions.
 *
 * Copyright (C) 2009  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARGPARSER_H__
#define __ARGPARSER_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <regex.h>
#include <getopt.h>

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>

#define VERSION "v0.7"
#define AUTHOR "Acri Emanuele (crossbower@gmail.com)"

#define MIN_ARG 256
#define MAX_ARG 1024

#define MAX_DATA 2048


/*
 * IP range structure
 */
struct iprange {
    char range[MIN_ARG];    // ip range
    char block[MIN_ARG];    // block
    char addr[MIN_ARG];     // address
    
    char http_request[MAX_DATA]; // http request
    int reqtype;   // request type
    
    int start;     // fist host
    int end;       // last host
    
    int start2;    // first net (only ipv4)
    int end2;      // last net  (only ipv4)
    
    int port;      // port
    int time;      // child lifetime
    
    int ipver;     // ip version
    
    int max_threads;   // max threads
    
    int range_ext:1;   // extended range (only ipv4)    
    int verbose:1;     // verbose
    int cookiejar:1;   // cookie-jar output
    int fingerprint:1; // fingerprint
};
#define VER_IPV4 4
#define VER_IPV6 6

/*
 * Request types
 */
enum requests_t {
    GET_REQ=1,
    HEAD_REQ,
    GET_NOT_REQ,
    GET_WRONG_REQ,
    OPTIONS_REQ,
    PUT_REQ,
    DELETE_REQ
};

/*
 * Variables used by argparser.c
 */
#ifdef ARGPARSER_C
#undef ARGPARSER_C

/*
 * Regular expression used by
 * parseopt() and parseipr() functions
 */
const char *reg_ipv4_range =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}[-][0-9]{1,3}$";

const char *reg_ipv4_range_ext =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}[-][0-9]{1,3}\\.[0-9]{1,3}$";

const char *reg_ipv4_addr =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}$";

const char *reg_ipv6_range =
    "^[0-9a-fA-F:]{1,42}[-][0-9a-fA-F]{1,4}$";

const char *reg_ipv6_addr =
    "^[0-9a-fA-F:]{1,42}$";

const char *reg_number =
    "\\d+?";

/*
 * Http 1.1 requests
 */
const char *get_request =
"GET";

const char *head_request =
"HEAD";

const char *get_not_exist_request =
"GET /foo.foo";

const char *get_wrong_version_request =
"GET / HTTP/9.9";

const char *options_request =
"OPTIONS";

const char *delete_request =
"DELETE";

/*
 * Program usage template
 */
const char *usage_template =
    "\nHTTSquash scanner " VERSION " - " AUTHOR "\n" \
    "Usage:\n" \
    "  httsquash -r range [options]\n" \
    "Options:\n" \
    "  -r\trange of ip addresses or target dns name\n" \
    "    \texamples: 208.67.1.1-254, 2001::1428:57ab-6344, google.com\n" \
    "  -p\tport (default 80)\n" \
    "Extra options:\n" \
    "  -t\ttime in seconds (default 3)\n" \
    "  -m\tmax scan threads (default 10)\n" \
    "  -v\tfull answer (include html data)\n" \
    "  -j\tcookie jar separator (\"%%\")\n" \
/*    "  -P\tpersonalized http request, payload file\n" \ //*/
    "  -T\trequest type (default get)\n" \
    "    \ttypes: get, head, delete\n" \
    "  -F\tenable fingerprinting (request type required)\n";

#endif /* ARGPARSER_C */

/*
 * Program usage
 */
void usage(FILE *out, const char *error);

/*
 * A personalized strncat
 */
char *myncat(char *dest, const char *src, size_t n);


/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char* str, const char* regex);

/*
 * Read http request from a file
 */
int readhttprequest(struct iprange *ipr, char *path);

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(struct iprange *ipr, int argc, char **argv);

#endif /* __ARGPARSER_H__ */

