/*
 * ARGPARSER.C
 * Usage and options parser...
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define ARGPARSER
#include "argparser.h"

/*
 * Program usage
 */
void usage(FILE *out, const char *error)
{
    fputs(usage_template, out);

    if(error)
        fputs(error, out);

    exit(1);
}

/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char *str, const char *regex)
{
    regex_t re;
    
    // Compile the regex pattern.
    if (regcomp(&re, regex, REG_EXTENDED | REG_NOSUB) != 0) {
        fprintf(stderr, "Error on regex pattern compilation.\n");
        return -1;
    }

    // Use the pattern on the passed string
    if (regexec(&re, str, 0, NULL, 0))  {
        regfree(&re);
        return 0;
    }
    
    regfree(&re);

    return 1;
}

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(struct options *opt, int argc, char **argv)
{
    char *c=NULL, *x=NULL;
    char ch;
    
    char *domain=NULL;   // domain
    char range[MIN_ARG]; // range
    char *wordlist=NULL; // wordlist
    int timeout=0;       // timeout
    int permutation=0;   // permutation
    int usetcp=0;        // use tcp queries
    int norecurse=0;     // no recursion
    
    int ipver=0;         // ip version
    int single=0;        // single ip?    
    
    // cleaning
    range[0]='\0';
    memset(opt, 0, sizeof(struct options));
    
    
    const char *shortopt = "d:r:w:t:PD:TR"; // short options
    
    while ((ch = getopt (argc, argv, shortopt)) != -1)
        switch (ch) {
        
            case 'd': // Domain
                domain = optarg;
                break;
            
            case 'r': // Range
            
                // ipv4 range
                if(regmatch(optarg, reg_ipv4_range)) {
                    ipver=VER_IPV4;
                }
                
                // ipv4 single
                else if(regmatch(optarg, reg_ipv4_addr)) {
                    ipver=VER_IPV4;
                    single=1;
                }
                
                // ipv6 range
                else if(regmatch(optarg, reg_ipv6_range)) {
                    ipver=VER_IPV6;
                }
                
                // ipv6 single
                else if(regmatch(optarg, reg_ipv6_addr)) {
                    ipver=VER_IPV6;
                    single=1;
                }
                
                else {
                    usage(stderr, "\nArgument range is invalid.\n");
                }
                
                strncpy(range, optarg, MIN_ARG);

                break;
            
            case 'w': // Wordlist
                wordlist = optarg;
                break;
            
            case 't': // Timeout
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument timeout is invalid.\n");
                timeout = atoi(optarg);
                break;
            
            case 'P': // Permutation
                permutation = 1;
                break;
            
            case 'D': // Nameserver
                strncpy(opt->nameserver, optarg, MAX_ARG);
                break;
            
            case 'T': // TCP queries
                usetcp = 1;
                break;
            
            case 'R': //no recursion
                norecurse = 1;
                break;
            
            case '?':
            default:
                usage(stderr, NULL);
        }

    //check collected args and fill options struct
    if(!domain && range[0]=='\0')
        usage(stderr, "\nDomain or range? Uff...\n");

    if(domain && !wordlist)
        usage(stderr, "\nYou forgot something (wordlist) :p\n");

    // domain
    if(domain) {
        strncpy(opt->domain, domain, MAX_ARG);
        strncpy(opt->wordlist, wordlist, MAX_ARG);
    }

    // fill options struct
    opt->timeout = timeout > 0 ? timeout : 0;
    opt->permutation = permutation;
    opt->usetcp = usetcp;
    opt->norecurse = norecurse;    
    
    // ip version
    opt->ipver = ipver;
    
    // range
	if(range[0]!='\0') {
	
        strncpy(opt->range, range, MIN_ARG);
        
        // ipv4
        if(ipver==VER_IPV4) {
            c = range;
            x = range;

            while(*c!='\0') c++;
            while(*c!='.') c--;

            *c = '\0';
            strncpy(opt->block, range, MIN_ARG);
            *c = '.';

            // start
            if(!single) {
                while(*x!='-') x++;
                *x='\0';
            }
            
            c++;
            
            opt->start=atoi(c);
    
            // single ip
            if(single) {
                strcat(opt->range,"-");
                strcat(opt->range,c);
                opt->end=opt->start;
            }
            
            // range
            else {
                x++;
                opt->end=atoi(x);
            }
        }
        
        // ipv6
        else if(ipver==VER_IPV6) {
            c = range;
            x = range;

            while(*c!='\0') c++;
            while(*c!=':') c--;

            *c = '\0';
            strncpy(opt->block, range, MIN_ARG);
            *c = ':';

            // start
            if(!single) {
                while(*x!='-') x++;
                *x='\0';
            }
            
            c++;
            
            sscanf(c, "%x", &opt->start);

            // single ip
            if(single) {
                strcat(opt->range,"-");
                strcat(opt->range,c);
                opt->end=opt->start;
            }
            
            // range
            else {
                x++;
                sscanf(x, "%x", &opt->end);
            }
        }
	}
	
    // check range
    if(opt->end < opt->start)
        usage(stderr, "\nArgument range is impossible ;)\n");
	
    return 1;
}
