/*
 * ARGPARSER.H
 * Usage and options parser...
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARGPARSER_H__
#define __ARGPARSER_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <getopt.h>
#include <regex.h>

#define VERSION "v0.7"
#define AUTHOR "Acri Emanuele (crossbower@gmail.com)"

#define MIN_ARG 256
#define MAX_ARG 1024

/*
 * Options structure
 */
struct options {
    char domain[MAX_ARG];     // domain
    char range[MIN_ARG];      // range
    char wordlist[MAX_ARG];   // wordlist
    int timeout;              // timeout
    int permutation;          // permutation
    int usetcp;               // use tcp queries
    int norecurse;            // no recursion
    
    char nameserver[MAX_ARG]; // nameserver
    
    int ipver;                // ip version
    
    int start;                // for reverse scanning
    int end;                  // ...
    char block[MIN_ARG];      // ...
};
#define VER_IPV4 4
#define VER_IPV6 6


/*
 * Only for argparser.c
 */
#ifdef ARGPARSER

/*
 * Regular expression used by
 * parseopt() function
 */
const char *reg_ipv4_range =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}[-][0-9]{1,3}$";

const char *reg_ipv4_addr =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}$";

const char *reg_ipv6_range =
    "^[0-9a-fA-F:]{1,42}[-][0-9a-fA-F]{1,4}$";

const char *reg_ipv6_addr =
    "^[0-9a-fA-F:]{1,42}$";

const char *reg_number =
    "\\d+?";

/*
 * Program usage template
 */
const char usage_template[] =
    "\nReverseRaider domain scanner " VERSION " - " AUTHOR "\n" \
    "Usage:\n" \
    "  reverseraider -d domain | -r range [options]\n" \
    "Options:\n" \
    "  -r\trange of ipv4 or ipv6 addresses, for reverse scanning\n" \
    "    \texamples: 208.67.1.1-254 or 2001:0DB8::1428:57ab-6344\n" \
    "  -d\tdomain, for wordlist scanning (example google.com)\n" \
    "  -w\twordlist file (see wordlists directory...)\n" \
    "Extra options:\n" \
    "  -t\trequests timeout in seconds\n" \
    "  -P\tenable numeric permutation on wordlist (default off)\n" \
    "  -D\tnameserver to use (default: resolv.conf)\n" \
    "  -T\tuse TCP queries instead of UDP queries\n" \
    "  -R\tdon't set the recursion bit on queries\n";

#endif /* ARGPARSER */

/*
 * Program usage
 */
void usage(FILE *out, const char *error);

/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char *str, const char *regex);

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(struct options *opt, int argc, char **argv);

#endif /* __ARGPARSER_H__ */

